// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file PetscSolverNavierStokes.h */

#ifndef AMDIS_PETSC_SOLVER_NAVIER_STOKES_H
#define AMDIS_PETSC_SOLVER_NAVIER_STOKES_H

#include "parallel/PetscSolverGlobalMatrix.h"

namespace AMDiS {

  using namespace std;

  struct NavierStokesSchurData {
    KSP kspMass, kspLaplace;
    
    Mat matConDif;
  };

  class PetscSolverNavierStokes : public PetscSolverGlobalMatrix
  {
  private:
    class IdFct : public AbstractFunction<double, double>
    {
    public:
      IdFct() 
	: AbstractFunction<double, double>(0)
      {}
      
      double operator()(const double& x) const 
      {
	return x;
      }
    };

  public:
    PetscSolverNavierStokes(string name);

    void setStokesData(double *nuPtr, double *invTauPtr, SystemVector *vec)
    {
      nu = nuPtr;
      invTau = invTauPtr;
      solution = vec;
    }

  protected:
    void initSolver(KSP &ksp);

    void initPreconditioner(PC pc);

  private:
    int pressureComponent;

    PetscSolver *massMatrixSolver, *laplaceMatrixSolver, *conDifMatrixSolver;

    NavierStokesSchurData matShellContext;

    double *nu, *invTau;

    SystemVector* solution;
    
    IdFct idFct;
  };

}

#endif
